@@ -36,14 +36,18 @@
 #include <gtk/gtk.h>
 #include <libintl.h>
 #include <conic.h>
+#include <dbus/dbus-glib.h>
+#include <osso-ic-dbus.h>
 
 #define HOME_DIR g_get_home_dir()
 
 #define IFCONFIG "/sbin/ifconfig "
 #define IP_COMMAND " | grep \"inet addr\" | awk -F: '{print $2}' | awk '{print $1}'"
-#define INTERFACE_COMMAND "/sbin/route -n | awk '$1 == \"0.0.0.0\" {print $8}'"
+//#define INTERFACE_COMMAND "/sbin/route -n | awk '$1 == \"0.0.0.0\" {print $8}'"
+#define INTERFACE_COMMAND "/sbin/route -n | awk '$1 == \"0.0.0.0\" && $3 == \"0.0.0.0\" {print $8}'"
 
-#define _(String) dgettext("hildon-libs", String)
+#define _tran(String) dgettext("hildon-libs", String)
+#define _(String) gettext (String)
 
 #define PERSONAL_IP_ADDRESS_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE (obj,\
 							                         PERSONAL_TYPE_IP_ADDRESS,\
@@ -51,20 +55,27 @@
 
 struct _PersonalIpAddressPrivate
 {
-    GtkWidget *homeWidget;
+	GtkWidget *homeWidget;
 	GtkWidget *event;
-    GtkWidget *contents;
+	GtkWidget *contents;
 	
 	GtkWidget *ipImage;
 	GtkWidget *ipLabel;
 	GtkWidget *ipContent;
 	GtkWidget *interfaceContent;
+	GtkWidget *networkContent;
 	
 	gboolean connectOnPress;
+	gboolean disconnectOnPress;
 	gboolean compactMode;
+	gboolean showInterface;
 	gboolean isPressed;
+	gboolean isConnected;
 	
 	ConIcConnection *connection;
+
+	DBusGConnection *dbus_connection;
+	DBusGProxy *dbus_icd2_proxy;
 	
 	guint eventID;
 };
@@ -92,6 +103,12 @@ void personal_ip_address_read_settings ( PersonalIpAddress *self )
             self->priv->connectOnPress = FALSE;
             g_error_free (error);
             error = NULL;
+        }
+        self->priv->disconnectOnPress = g_key_file_get_boolean (keyFile, "config", "disconnectOnPress", &error);
+        if (error) {
+            self->priv->disconnectOnPress = FALSE;
+            g_error_free (error);
+            error = NULL;
         }
 		self->priv->compactMode = g_key_file_get_boolean (keyFile, "config", "compactMode", &error);
 		if (error) {
@@ -99,9 +116,17 @@ void personal_ip_address_read_settings ( PersonalIpAddress *self )
 			g_error_free (error);
 			error = NULL;
 		}
+        self->priv->showInterface = g_key_file_get_boolean (keyFile, "config", "showInterface", &error);
+        if (error) {
+            self->priv->showInterface = TRUE;
+            g_error_free (error);
+            error = NULL;
+        }
     } else {
         self->priv->connectOnPress = FALSE;
+        self->priv->disconnectOnPress = FALSE;
 		self->priv->compactMode = TRUE;
+		self->priv->showInterface = TRUE;
     }
 
     g_key_file_free (keyFile);
@@ -120,7 +145,9 @@ void personal_ip_address_write_settings (PersonalIpAddress *self)
 	keyFile = g_key_file_new();
 		
 	g_key_file_set_boolean (keyFile, "config", "connectOnPress", self->priv->connectOnPress);
+	g_key_file_set_boolean (keyFile, "config", "disconnectOnPress", self->priv->disconnectOnPress);
 	g_key_file_set_boolean (keyFile, "config", "compactMode", self->priv->compactMode);
+	g_key_file_set_boolean (keyFile, "config", "showInterface", self->priv->showInterface);
 	
 	filename = g_strconcat (HOME_DIR, "/.personal_ip_address", NULL);
 	fileData = g_key_file_to_data (keyFile, &size, NULL);
@@ -147,7 +174,10 @@ void personal_ip_address_button_release (GtkWidget *widget, GdkEventButton *even
 	if (self->priv->connectOnPress) {
 		self->priv->isPressed = FALSE;
 	
-		con_ic_connection_connect (self->priv->connection, CON_IC_CONNECT_FLAG_NONE);
+		if (self->priv->isConnected && self->priv->disconnectOnPress)
+			system ("run-standalone.sh dbus-send --system --dest=com.nokia.icd /com/nokia/icd_ui com.nokia.icd_ui.disconnect boolean:true");
+		else
+			con_ic_connection_connect (self->priv->connection, CON_IC_CONNECT_FLAG_NONE);
 	
 		gtk_widget_queue_draw (GTK_WIDGET (self));
 	}
@@ -178,21 +208,27 @@ void personal_ip_address_content_create (PersonalIpAddress *self)
 		self->priv->ipImage = gtk_image_new_from_pixbuf (pixbuf);
 		g_object_unref (pixbuf);
 	}
-	self->priv->ipLabel = gtk_label_new ("IP");
+	self->priv->ipLabel = gtk_label_new (_("IP"));
 	GtkWidget *box = gtk_hbox_new (FALSE, 0);
+	self->priv->networkContent = gtk_label_new ("");
 	self->priv->interfaceContent = gtk_label_new ("");
-	self->priv->ipContent = gtk_label_new ("(no IP found)");
+	self->priv->ipContent = gtk_label_new (_("(no IP found)"));
+	hildon_helper_set_logical_font (self->priv->networkContent, "HomeSystemFont");
+	gtk_label_set_ellipsize (GTK_LABEL (self->priv->networkContent), PANGO_ELLIPSIZE_END);
 	gtk_box_pack_start (GTK_BOX (box), self->priv->ipImage, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (box), self->priv->ipLabel, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (box), self->priv->interfaceContent, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (box), self->priv->ipContent, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (box), self->priv->interfaceContent, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (box), self->priv->networkContent, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (box), self->priv->ipContent, FALSE, FALSE, 0);
 	hildon_helper_set_logical_color (self->priv->ipContent, GTK_RC_FG, GTK_STATE_NORMAL, "ActiveTextColor");
 	hildon_helper_set_logical_color (self->priv->interfaceContent, GTK_RC_FG, GTK_STATE_NORMAL, "ActiveTextColor");
 	gtk_misc_set_alignment (GTK_MISC (self->priv->ipLabel), 0, 1);
 	gtk_misc_set_alignment (GTK_MISC (self->priv->ipContent), 1, 1);
 	gtk_misc_set_alignment (GTK_MISC (self->priv->interfaceContent), 0, 1);
+	gtk_misc_set_alignment (GTK_MISC (self->priv->networkContent), 0, 1);
 	gtk_misc_set_padding (GTK_MISC (self->priv->ipContent), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
 	gtk_misc_set_padding (GTK_MISC (self->priv->interfaceContent), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
+	gtk_misc_set_padding (GTK_MISC (self->priv->networkContent), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
 	gtk_misc_set_padding (GTK_MISC (self->priv->ipLabel), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
 
 	gtk_container_add (GTK_CONTAINER (self->priv->contents), box);
@@ -205,10 +241,44 @@ void personal_ip_address_content_create (PersonalIpAddress *self)
     gtk_widget_show_all (self->priv->homeWidget);
 	if (self->priv->compactMode) {
 		gtk_widget_hide (self->priv->ipImage);
-		gtk_widget_show (self->priv->ipLabel);
+		if (self->priv->showInterface) {
+			gtk_widget_show (self->priv->ipLabel);
+			gtk_widget_show (self->priv->interfaceContent);
+			gtk_widget_hide (self->priv->networkContent);
+		} else {
+			gtk_widget_hide (self->priv->ipLabel);
+			gtk_widget_hide (self->priv->interfaceContent);
+			gtk_widget_show (self->priv->networkContent);
+		}
 	} else {
 		gtk_widget_hide (self->priv->ipLabel);
 		gtk_widget_show (self->priv->ipImage);
+		if (self->priv->showInterface) {
+			gtk_widget_show (self->priv->interfaceContent);
+			gtk_widget_hide (self->priv->networkContent);
+		} else {
+			gtk_widget_hide (self->priv->interfaceContent);
+			gtk_widget_show (self->priv->networkContent);
+		}
+	}
+}
+
+static void personal_ip_address_on_method_call_finished (DBusGProxy *proxy, DBusGProxyCall *call_id, PersonalIpAddress *self)
+{
+	gchar *network_id = NULL;
+	guint active_time, signal_strength, packets_recieved, packets_sent, bytes_recieved, bytes_sent;
+
+	dbus_g_proxy_end_call (proxy, call_id, NULL, G_TYPE_STRING, &network_id, G_TYPE_UINT, &active_time, G_TYPE_UINT, &signal_strength, G_TYPE_UINT, &packets_recieved, G_TYPE_UINT, &packets_sent, G_TYPE_UINT, &bytes_recieved, G_TYPE_UINT, &bytes_sent, G_TYPE_INVALID);
+	if (network_id)
+	{
+		ConIcIap *iap = con_ic_connection_get_iap (self->priv->connection, network_id);
+		const gchar *connection_name = con_ic_iap_get_name (iap);
+		//gchar *final_name = g_strconcat (connection_name, " (", gtk_label_get_text (GTK_LABEL (self->priv->interfaceContent)), ")", NULL);
+		//if (connection_name)
+			gtk_label_set_text (GTK_LABEL (self->priv->networkContent), connection_name);
+		g_object_unref (iap);
+		g_free (network_id);
+		//g_free (final_name);
 	}
 }
 
@@ -248,8 +318,15 @@ gboolean personal_ip_address_update_content (PersonalIpAddress *self)
 	
 	if (!found) {
 		gtk_label_set_text (GTK_LABEL (self->priv->interfaceContent), "");
-		gtk_label_set_text (GTK_LABEL (self->priv->ipContent), "(no IP found)");
+		gtk_label_set_text (GTK_LABEL (self->priv->networkContent), "");
+		gtk_label_set_text (GTK_LABEL (self->priv->ipContent), _("(no IP found)"));
 	}
+	else
+	{
+		if (self->priv->dbus_icd2_proxy)
+			dbus_g_proxy_begin_call (self->priv->dbus_icd2_proxy, ICD_GET_STATISTICS_REQ, (DBusGProxyCallNotify) personal_ip_address_on_method_call_finished, self, NULL, G_TYPE_INVALID);
+	}
+	self->priv->isConnected = found;
 }
 
 gboolean personal_ip_address_connection_changed (ConIcConnection *connection, ConIcConnectionEvent *event, PersonalIpAddress *self)
@@ -259,40 +336,84 @@ gboolean personal_ip_address_connection_changed (ConIcConnection *connection, Co
 	return TRUE;
 }
 
+void personal_ip_address_connectOnPress_changed (HildonCheckButton *button, GtkWidget *target)
+{
+	gtk_widget_set_sensitive (target, hildon_check_button_get_active (button));
+}
+
 void personal_ip_address_settings (HDHomePluginItem *hitem, PersonalIpAddress *self)
 {
 	//g_warning ("personal_ip_address_settings");
-	GtkWidget *dialog = gtk_dialog_new_with_buttons ("Personal IP Address Settings",
-        NULL, 0, _("wdgt_bd_save"), GTK_RESPONSE_ACCEPT, NULL);
-    GtkWidget *content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+	GtkWidget *dialog = gtk_dialog_new_with_buttons (_("Personal IP Address Settings"),
+        NULL, 0, _tran("wdgt_bd_save"), GTK_RESPONSE_ACCEPT, NULL);
+	GtkWidget *content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 	GtkSizeGroup *group = GTK_SIZE_GROUP (gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL));
 	
 	GtkWidget *connectOnPress = hildon_check_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
-	gtk_button_set_label (GTK_BUTTON (connectOnPress), "Connect to network when pressed");
+	gtk_button_set_label (GTK_BUTTON (connectOnPress), _("Connect to network when pressed"));
 	gtk_container_add (GTK_CONTAINER (content_area), connectOnPress);
 	hildon_check_button_set_active (HILDON_CHECK_BUTTON (connectOnPress), self->priv->connectOnPress);
 	
+	GtkWidget *disconnectOnPress = hildon_check_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
+	gtk_button_set_label (GTK_BUTTON (disconnectOnPress), _("Disconnect from network when pressed"));
+	gtk_container_add (GTK_CONTAINER (content_area), disconnectOnPress);
+	hildon_check_button_set_active (HILDON_CHECK_BUTTON (disconnectOnPress), self->priv->disconnectOnPress);
+	gtk_widget_set_sensitive (disconnectOnPress, self->priv->connectOnPress);
+	g_signal_connect (connectOnPress, "toggled", G_CALLBACK (personal_ip_address_connectOnPress_changed), disconnectOnPress);
+		
 	GtkWidget *compactMode = hildon_check_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
-	gtk_button_set_label (GTK_BUTTON (compactMode), "Use compact layout");
+	gtk_button_set_label (GTK_BUTTON (compactMode), _("Use compact layout"));
 	gtk_container_add (GTK_CONTAINER (content_area), compactMode);
 	hildon_check_button_set_active (HILDON_CHECK_BUTTON (compactMode), self->priv->compactMode);
 	
+	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
+	
+	GtkWidget *showInterface = hildon_gtk_radio_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, NULL);
+	gtk_button_set_label (GTK_BUTTON (showInterface), _("Show Network Interface"));
+	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (showInterface), FALSE);
+	gtk_box_pack_start (GTK_BOX (hbox), showInterface, TRUE, TRUE, 0);
+	
+	GtkWidget *showNetwork = hildon_gtk_radio_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, gtk_radio_button_get_group (GTK_RADIO_BUTTON (showInterface)));
+	gtk_button_set_label (GTK_BUTTON (showNetwork), _("Show Network Name"));
+	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (showNetwork), FALSE);
+	gtk_box_pack_start (GTK_BOX (hbox), showNetwork, TRUE, TRUE, 0);
+	gtk_container_add (GTK_CONTAINER (content_area), hbox);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (showInterface), self->priv->showInterface);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (showNetwork), !self->priv->showInterface);
+	
 	gtk_widget_show_all (dialog);
 	
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
 		self->priv->connectOnPress = hildon_check_button_get_active (HILDON_CHECK_BUTTON (connectOnPress));
+		self->priv->disconnectOnPress = hildon_check_button_get_active (HILDON_CHECK_BUTTON (disconnectOnPress));
 		self->priv->compactMode = hildon_check_button_get_active (HILDON_CHECK_BUTTON (compactMode));
+		self->priv->showInterface = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (showInterface));
 		
 		if (self->priv->compactMode) {
 			gtk_widget_hide (self->priv->ipImage);
-			gtk_widget_show (self->priv->ipLabel);
-			gtk_widget_set_size_request (GTK_WIDGET (self), 315, 40);
-			gtk_window_resize (GTK_WINDOW (self), 315, 40);
+			if (self->priv->showInterface) {
+				gtk_widget_show (self->priv->ipLabel);
+				gtk_widget_show (self->priv->interfaceContent);
+				gtk_widget_hide (self->priv->networkContent);
+			} else {
+				gtk_widget_hide (self->priv->ipLabel);
+				gtk_widget_hide (self->priv->interfaceContent);
+				gtk_widget_show (self->priv->networkContent);
+			}
+			gtk_widget_set_size_request (GTK_WIDGET (self), 352, 40);
+			gtk_window_resize (GTK_WINDOW (self), 352, 40);
 		} else {
 			gtk_widget_hide (self->priv->ipLabel);
 			gtk_widget_show (self->priv->ipImage);
-			gtk_widget_set_size_request (GTK_WIDGET (self), 315, 56);
-			gtk_window_resize (GTK_WINDOW (self), 315, 56);
+			if (self->priv->showInterface) {
+				gtk_widget_show (self->priv->interfaceContent);
+				gtk_widget_hide (self->priv->networkContent);
+			} else {
+				gtk_widget_hide (self->priv->interfaceContent);
+				gtk_widget_show (self->priv->networkContent);
+			}
+			gtk_widget_set_size_request (GTK_WIDGET (self), 352, 56);
+			gtk_window_resize (GTK_WINDOW (self), 352, 56);
 		}
 		
 		personal_ip_address_write_settings (self);
@@ -336,6 +457,9 @@ static void personal_ip_address_finalize (GObject *object)
 	}
 	g_object_unref (self->priv->connection);
 
+	if (self->priv->dbus_icd2_proxy)
+		g_object_unref (self->priv->dbus_icd2_proxy);
+
 	G_OBJECT_CLASS (personal_ip_address_parent_class)->finalize (object);
 }
 
@@ -430,38 +554,47 @@ static void personal_ip_address_class_finalize (PersonalIpAddressClass *klass G_
 static void personal_ip_address_init (PersonalIpAddress *self)
 {
 	//g_warning ("personal_ip_address_init");
+	setlocale (LC_ALL, "");
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+	textdomain (GETTEXT_PACKAGE);
+
 	self->priv = PERSONAL_IP_ADDRESS_GET_PRIVATE (self);
 	self->priv->eventID = 0;
 	self->priv->isPressed = FALSE;
+	self->priv->isConnected = FALSE;
 	
 	self->priv->connection = con_ic_connection_new ();
 	g_signal_connect (self->priv->connection, "connection-event", G_CALLBACK (personal_ip_address_connection_changed), self);
 	g_object_set (self->priv->connection, "automatic-connection-events", TRUE);
+
+	self->priv->dbus_connection = hd_home_plugin_item_get_dbus_g_connection (&self->parent, DBUS_BUS_SYSTEM, NULL);
+	if (self->priv->dbus_connection)
+		self->priv->dbus_icd2_proxy = dbus_g_proxy_new_for_name (self->priv->dbus_connection, ICD_DBUS_SERVICE, ICD_DBUS_PATH, ICD_DBUS_INTERFACE);
 	
 	hd_home_plugin_item_set_settings (&self->parent, TRUE);
 	g_signal_connect (&self->parent, "show-settings", G_CALLBACK (personal_ip_address_settings), self);
 
-	gtk_window_set_default_size (GTK_WINDOW (self), 315, 56);
+	gtk_window_set_default_size (GTK_WINDOW (self), 352, 56);
 	
 	self->priv->homeWidget = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (self), self->priv->homeWidget);
 	gtk_widget_show (self->priv->homeWidget);
 
 	GdkGeometry hints;
-	hints.min_width = 315;
+	hints.min_width = 352;
 	hints.min_height = 40;
-	hints.max_width = 315;
+	hints.max_width = 352;
 	hints.max_height = 56;
 	gtk_window_set_geometry_hints (GTK_WINDOW (self), self->priv->homeWidget, &hints, GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE);
 	
 	personal_ip_address_read_settings (self);
 	
 	if (self->priv->compactMode) {
-		gtk_widget_set_size_request (GTK_WIDGET (self), 315, 40);
-		gtk_window_resize (GTK_WINDOW (self), 315, 40);
+		gtk_widget_set_size_request (GTK_WIDGET (self), 352, 40);
+		gtk_window_resize (GTK_WINDOW (self), 352, 40);
 	} else {
-		gtk_widget_set_size_request (GTK_WIDGET (self), 315, 56);
-		gtk_window_resize (GTK_WINDOW (self), 315, 56);
+		gtk_widget_set_size_request (GTK_WIDGET (self), 352, 56);
+		gtk_window_resize (GTK_WINDOW (self), 352, 56);
 	}
 	
 	personal_ip_address_content_create (self);
